/*
 * Copyright © 2013 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *     Antti Kaijanmäki <antti.kaijanmaki@canonical.com>
 */

#include <iostream>
#include <memory>

#include <signal.h>

#include <com/ubuntu/connectivity/networking/manager.h>

namespace cuc = com::ubuntu::connectivity;

int
main(int argc, char *argv[])
{
    auto mgr = cuc::networking::Manager::createInstance();

    // Subscribe to system networking changes
    mgr->status().changed().connect(
                [](cuc::networking::Manager::NetworkingStatus status)
    {
        switch(status) {
        case cuc::networking::Manager::NetworkingStatus::offline:
        {
            std::cout << "System networking status changed to: offline" << std::endl;
            break;
        }
        case cuc::networking::Manager::NetworkingStatus::connecting:
        {
            std::cout << "System networking status changed to: connecting" << std::endl;
            break;
        }
        case cuc::networking::Manager::NetworkingStatus::online:
        {
            std::cout << "System networking status changed to: online" << std::endl;
            break;
        }
        }
    });

    mgr->charasteristics().changed().connect(
                [](int charasteristics)
    {
        std::cout << "System networking charasteristics changed:" << std::endl;
        if ((charasteristics & cuc::networking::Link::Characteristics::has_monetary_costs) != 0) {
            std::cout << "    - has monetary costs" << std::endl;
        }
        if ((charasteristics & cuc::networking::Link::Characteristics::is_bandwidth_limited) != 0) {
            std::cout << "    - is bandwith limited" << std::endl;
        }

        if ((charasteristics & cuc::networking::Link::Characteristics::is_volume_limited) != 0) {
            std::cout << "    - is volume_limited" << std::endl;
        }
    });

    sigset_t signal_set;
    sigemptyset(&signal_set);
    sigaddset(&signal_set, SIGINT);
    int signal;
    sigwait(&signal_set, &signal);

    return 0;
}
