/*
 * Copyright © 2013 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *     Antti Kaijanmäki <antti.kaijanmaki@canonical.com>
 */

#ifndef COM_UBUNTU_CONNECTIVITY_NETWORKING_LINK
#define COM_UBUNTU_CONNECTIVITY_NETWORKING_LINK

#include <core/property.h>

namespace com {
namespace ubuntu {
namespace connectivity {
namespace networking {

class __attribute ((visibility ("default")))
Link
{
public:
    Link(const Link&) = delete;
    virtual ~Link() = default;

    Link& operator=(const Link&) = delete;
    bool operator==(const Link&) const = delete;

    enum class Type
    {
        wifi,
        wired,
        wwan,
        service
    };

    enum class Status
    {
        disabled,
        offline,
        connecting,  // link is in process of becoming online
        connected,   // the link is up, but not fully configured yet
                     // happens with wifi for example when captive portal
                     // login is required
        online
    };

    enum Characteristics : int
    {
        empty = 0,
        has_monetary_costs = 1 << 0,
        is_volume_limited = 1 << 1,
        is_bandwidth_limited = 1 << 2
    };

    virtual void enable()  = 0;
    virtual void disable() = 0;

    virtual Type type() const = 0;
    virtual const core::Property<int>& charasteristics() = 0;

    virtual const core::Property<Status>& status() = 0;

    typedef unsigned int Id;
    virtual Id id() const = 0;

    virtual std::string name() const = 0;

protected:
    Link() = default;
};

}
}
}
}

#endif
