/*
 * Copyright © 2013 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *     Antti Kaijanmäki <antti.kaijanmaki@canonical.com>
 */

#ifndef COM_UBUNTU_CONNECTIVITY_NETWORKING_MANAGER
#define COM_UBUNTU_CONNECTIVITY_NETWORKING_MANAGER

#include <com/ubuntu/connectivity/networking/link.h>
#include <com/ubuntu/connectivity/networking/service.h>

#include <core/property.h>
#include <core/signal.h>

#include <memory>

namespace com {
namespace ubuntu {
namespace connectivity {
namespace networking {

class __attribute ((visibility ("default")))
Manager
{
public:
    Manager &operator=(const Manager&) = delete;
    virtual ~Manager()                 = default;
    Manager(const Manager&)            = delete;

    static std::unique_ptr<Manager> createInstance();

    enum class FlightModeStatus {
        on,
        off
    };

    enum class NetworkingStatus {
        offline,
        connecting,
        online
    };

    virtual void enableFlightMode()  = 0;
    virtual void disableFlightMode() = 0;

    //! unprivileged
    virtual const core::Property<FlightModeStatus>& flightMode() = 0;

    virtual const core::Property<std::set<std::shared_ptr<Link>>>&    links() = 0;
    virtual const core::Property<std::set<std::shared_ptr<Service>>>& services() = 0;

    /* status of the overall system networking */
    virtual const core::Property<NetworkingStatus> &status() = 0;

    /* charasteristics of the overall system networking */
    virtual const core::Property<int>& charasteristics() = 0;

protected:
    Manager();
};

}
}
}
}

#endif
